#define DIV_EPS 0.0001

inline int sign(double x)
{
  if(fabs(x)<0.0001) return 0;
  else if(x>0) return 1;
  return -1;
}

inline double m_acos(double x)
{
  if(x>=1) return 0;
  if(x<=-1) return M_PI;
  return acos(x);
}

inline double m_asin(double x)
{
  if(x>=1) return M_PI_2;
  if(x<=-1) return -M_PI-2;
  return asin(x);
}

/////////////////////////////////////////////////////////////////
//	TEnemyBomber
//	 
//	   
class BomberAirfield;
class GroundObject;
class AccompFighter;
class EnemyBomber: public Aircraft
{
  typedef std::auto_ptr<std::vector<AccompFighter*> > PFighterList;
public:
	EnemyBomber(BomberAirfield* ba, TTH &_tth);

	double getPotentialDamage() { return has_bombs?tth.get("potential_damage_full"):
                      tth.get("potential_damage_empty"); }
  void SetTarget(TGroundObject*, const Vector3D&);
  void SetFighters(PFighterList*);
  TGroundObject* GetTarget(){return target;}
  int Ready() { return ready; }
  int Flying() { return flying; }
  void GoHome();
  void SetFighters(PFighterList);
  void OnDeleteObject(TWarObject*);
  static void __stdcall OnBackHome(TWarObject* __this, int);
  static void __stdcall OnDropBombs(TWarObject* __this, int);
  static void __stdcall OnEndMovingAside(TWarObject* __this, int);
  void RadarSawYou(TRadar *radar);
private:
  u32 event_id;
  void OnMove();
  void DropBombs();
	TBomberAirfield *airfield;

  Vector3D target_coord; // ,    
  TGroundObject* target;

  PFighterList fighters;
  double fighterR;

  double bombR, bombRalpha, bombDamage, bombSigma;
  //   
  double start_height;
  //  
  double end_height;
  // 
  double hV;
  // 
  double V;
  //     
  int ready;
  //
  int flying;
  //    
  int has_bombs;
  // 
  int gohome;
  int bMovingAside;
};


/////////////////////////////////////////////////////////////////
//	TEnemyFighter
//	 
//	   
//	  ""  
class AccompFighter: public Aircraft
{
public:
	AccompFighter(BomberAirfield* ba, TTH &_tth);

	double getPotentialDamage() { return tth.get("potential_damage"); }
  int Ready() { return ready; }
  int Flying() { return flying; }
  void GoHome();

  static void __stdcall OnBackHome(WarObject* __this, int);
  static void __stdcall OnLaunchRocket(WarObject* __this, int);
  static void __stdcall OnRocketFire(WarObject* __this, int);

  void OnDeleteObject(WarObject *obj);
  void SetBomber(EnemyBomber *);

private:
	BomberAirfield *airfield;
  Rocket *rocket;

  EnemyBomber* bomber;
  //     
  int ready;
  //
  int flying;
  //  
  int rockets;
  //   
  double rocketV, rocketProb, rocketR,rocketRmax,rocketRalpha, maxRocketT,
        rocketsMax, minH, maxH, ViewTime;
  Vector3D pos_sigma, speed_sigma;
  friend class EnemyBomber;
};


class Aircraft: public MovingObject
{
public:
	Aircraft(const Vector3D& pos0, const Vector3D speed0, TTH &_tth = TTH::defaultTTH);

  //         p   
	virtual void GoTo(const Vector3D& p);
	virtual void OnMove();
protected:
  void AddTurn(double, double);
  void AddLine(const Vector3D&);

  TPoints points;
	int isGoTo;
  double maxa;
	Vector3D dest;
};



/////////////////////////////////////////////////////////////////
