#include "atack.h"
#include "atackAirdrom.h"
#include "vector.h"

void Aircraft::GoTo(const Vector3D &p)
{
  Vector3D pos0=position;
  Vector3D pos1=p;
  //      
  //pos1.z=position.z;
  pos1.z=pos0.z=0;

//    !
  if(speedmod<DIV_EPS) return;

  //   
  Vector3D vnorm=speed;
  vnorm.z=0;
  vnorm.normalize();

  //         
  Vector3D dest=pos1-pos0;
  float ro=dest.length();
  if(ro<DIV_EPS) return;
  dest/=ro;

  //       
  float cosa=vnorm&dest;
  if(fabs(cosa)>1)
    cosa=sign(cosa);
  float sina=sqrt(1-cosa*cosa)*sign((vnorm^dest).z);

  //     
  //    . 
  float d=ro*sina;
  float ad=fabs(d);
  float l=ro*cosa;

  //    
  Vector3D turn_center;
  int angle_sign=-sign(d);
  if(angle_sign==0)
     angle_sign=(sign(Random()-0.5)>0?1:-1);
  turn_center=pos0+R*Vector3D(vnorm.y, -vnorm.x, 0)*angle_sign;

  //     ,  ,  
  //     
  if(cosa>0.01 && (turn_center-p).length() > R)
  {
    float adr=ad-R;
    if(adr>0)
    {
      float m=sqrt(l*l+adr*adr);
      //.. m>R,       m
      float turn_angle=(M_PI-m_acos(adr/m)-m_acos(R/m))*angle_sign;
      AddTurn(turn_angle, R);
    }
    else
    {
      adr=-adr;
      float m=sqrt(l*l+d*d);

      float turn_angle;
      //,   ,  l ,    
      //  cosa>0.01
      if(l>DIV_EPS)
        turn_angle=(M_PI_2-m_acos(R/m)-atan(adr/l))*angle_sign;
      else
        turn_angle=(M_PI_2-m_acos(R/m)-M_PI)*angle_sign;
      AddTurn(turn_angle, R);
      AddLine(p);
    }
  }
  else
  { //     ,    
    //  
    if((turn_center-p).length()<R)
    {
      if(cosa>0)
      {
        pos0+=vnorm*l;
        turn_center+=vnorm*l;
      }
      pos0+=vnorm*R;
      turn_center+=vnorm*R;
      AddLine(pos0);
    }
    //      
    Vector3D rp=p-turn_center;
    float m=rp.length();//  m   > R

    //          
    //    
    float a=m_acos(R/m);

    Vector3D p0r=pos0-turn_center;
    //,   ,  ,    
    float b=0;
    if(p0r.length()>DIV_EPS)
      b=m_acos((p0r&rp)/p0r.length()/m);
    //     
    float turn_angle=(2*M_PI-a-b)*angle_sign;
    AddTurn(turn_angle, R);
  }
  AddLine(p);
  Tircraft::dest=p;
}

//       angle
// angle>0,   ,   
//
void Aircraft::AddTurn(float angle, float R)
{
  ControlPoint p;
  if(points.empty())
  {
    p.time=dispatcher->GetTime();
    p.pos=position;
    p.speed=speed;
  }
  else p=points.back();

  Vector3D vnorm=p.speed;
  vnorm.z=0;
  vnorm.normalize();
  Vector3D turn_center=p.pos+R*Vector3D(vnorm.y, -vnorm.x, 0)*sign(angle);
  Vector3D dp0=(p.pos-turn_center);

  float turn_length=fabs(R*angle);

  if(speed.length()<DIV_EPS) return;// 

  int times=turn_length/speed.length()/TURN_TICK;
  if(!times) return;
  float dangle=angle/times;
  for(int i=1;i<=times;i++)
  {
    ControlPoint p1;
    p1.pos=turn_center+dp0.rotateXY(-dangle*i);
    p1.speed=p.speed.rotateXY(-dangle*i);
    p1.time=p.time+TURN_TICK*i;
    points.push(p1);
  }
}

struct ControlPoint
{
  Vector3D pos, speed;
  u32 time;
  int operator < (const ControlPoint& p)const{return time<p.time;}
};
typedef std::queue<ControlPoint> Points;

//     
void Aircraft::AddLine(const Vector3D& pos1)
{
  ControlPoint p;
  if(points.empty())
  {
    p.time=dispatcher->GetTime();
    p.pos=position;
    p.speed=speed;
  }
  else p=points.back();

  Vector3D d=pos1-p.pos;
  d.z=0;

  if(speed.length()<DIV_EPS)
   return;
  int times=d.length()/speed.length()/TURN_TICK;
  if(times)
  {
    Vector3D vnorm=pos1-p.pos;
    vnorm.z=0;
    vnorm.normalize();
    vnorm*=speed.length();
    Vector3D dp=d/times;
    for(int i=1;i<times;i++)
    {
      ControlPoint p1;
      p1.pos=p.pos+dp*i;
      p1.time=p.time+TURN_TICK*i;
      p1.speed=vnorm;
      points.push(p1);
    }
  }
  p.pos=pos1;
//  p.speed=0;
  p.time+=d.length()/speed.length();
  points.push(p);
  return;
}

////////////////////////////////////////////////////////////////
//                                                            //
//                TEnemyBomber                                //
//                                                            //
////////////////////////////////////////////////////////////////
EnemyBomber::EnemyBomber(BomberAirfield* ba, TTH &_tth):
                          Aircraft(0, 0, _tth), airfield(ba)
{
  target=0;
  ready=1;
	position=airfield->Position();
  flying=0;
  gohome=0;
  event_id=0;
  has_bombs=1;

  fighterR=tth.get("fighterR", 2);
  bombR=tth.get("bombR", 1);
  bombRalpha=tth.get("bombRalpha", 1);
  bombDamage=tth.get("bombDamage", 2);
  bombSigma=tth.get("bombSigma", 1);
}

//SetTarget
//    -
void EnemyBomber::SetTarget(GroundObject* _target, const Vector3D& p)
{
  //   ,      
  if(flying)
  {
    while(!points.empty()) points.pop();
  }

  //    
  target=_target;
  target_coord=p;

  //   ,    ,   .
  if(!flying)
  {
    //    
    V=(tth.get("minV")+Random()*(tth.get("maxV")-tth.get("minV")))/TIME_FACTOR;
    hV=tth.get("maxVertV")*Random()/TIME_FACTOR;
    start_height=tth.get("minH")+Random()*(tth.get("maxH")-tth.get("minH"));
    end_height=tth.get("minH")+Random()*(tth.get("maxH")-tth.get("minH"));

		// ,     
		bMovingAside=1;
  	float asideAngle=(Random()-0.5)*tth.get("AsideAngle");
		Vector3D v=p-position;
		v.z=0;
		float l=v.length();
		v=v.rotateXY(asideAngle);
		v.normalize();
		v*=l*cos(asideAngle);

    //  
    position.z=start_height;
    speed=v;
    speed.z=0;
    speed.normalize();
    speed*=V;
    GoTo(v+position);
    has_bombs=1;
    event_id=dispatcher->AddEvent(points.back().time-dispatcher->GetTime(),
                               &OnEndMovingAside, this, 0, "end_going_aside");
  }
  else
  {
    if(event_id)
      dispatcher->DeleteEvent(event_id);
    event_id=0;
  	GoTo(target_coord);
    event_id=dispatcher->AddEvent(points.back().time-dispatcher->GetTime(),
                               &OnDropBombs, this, 0, "drop_bombs");
  	bMovingAside=0;
  }
  //-------------------------------------------
  ready=0;
  flying=1;
}

/////////////////////////////////////////////////////////////////////
void EnemyBomber::OnMove()
{
  Aircraft::OnMove();
  //  
  if(fighters.get() && fighters->size()>0 && speed.length()>DIV_EPS)
  {
    size_t f=fighters->size();
    Vector3D s=speed;
    s.normalize();
    s*=fighterR;
    float angle=M_PI*2/f;
    for(size_t i=0;i<f;i++)
    {
      AccompFighter *af=(*fighters)[i];
      af->position=position+s;
      af->speed=speed;
      s=s.rotateXY(angle);
    }
  }
}

//  -   
void __stdcall EnemyBomber::OnBackHome(WarObject* __this, int)
{
  EnemyBomber* _this=dynamic_cast<EnemyBomber*>(__this);
  _this->event_id=0;
  _this->ready=1;
  _this->has_bombs=1;
  _this->speed=0;
  _this->gohome=0;
  _this->flying=0;
  _this->airfield->OnBomberReturn();
}

//   -  
void __stdcall EnemyBomber::OnDropBombs(WarObject* __this, int)
{
  EnemyBomber* _this=dynamic_cast<EnemyBomber*>(__this);
  _this->event_id=0;
  _this->DropBombs();
  _this->GoHome();
}

void EnemyBomber::RadarSawYou(Radar *radar)
{
  if(bMovingAside&&!gohome)
  {
    dispatcher->DeleteEvent(event_id);
    event_id=0;

    position.z=end_height;

    GoTo(target_coord);
    event_id=dispatcher->AddEvent(points.back().time-dispatcher->GetTime(),
                               &OnDropBombs, this, 0, "drop_bombs");
    radar->Maneuvr(this);
    if(fighters.get())
      for(std::vector<TAccompFighter*>::iterator i=fighters->begin();i!=fighters->end();i++)
        radar->Maneuvr(*i);
  }
}

void __stdcall EnemyBomber::OnEndMovingAside(WarObject* __this, int)
{
  EnemyBomber* _this=dynamic_cast<EnemyBomber*>(__this);
  _this->event_id=0;
  _this->bMovingAside=0;
  _this->GoTo(_this->target_coord);

  _this->event_id=dispatcher->AddEvent(_this->points.back().time-dispatcher->GetTime(),
                               &OnDropBombs, _this, 0, "drop_bombs");
}

void EnemyBomber::OnDeleteObject(WarObject *obj)
{
  if(!fighters.get()) return;
  for(std::vector<TAccompFighter*>::iterator i=fighters->begin();i!=fighters->end();i++)
  {
    if((*i)==obj)
    {
      fighters->erase(i);
      break;
    }
  }
}

//////////////////////////////////////////////////////////////////
//   
void EnemyBomber::DropBombs()
{
  if(!has_bombs) return;
  has_bombs=0;

  airfield->BombDropped();
  Vector3D dr=norm_raspr(Vector3D(bombSigma, bombSigma, 0));

  for (WarObjectsList::iterator i = WarObjects.begin(); i != WarObjects.end(); )
  {
    GroundObject *obj = dynamic_cast<GroundObject *>(*i);
    i++;

    if (!obj)  continue;
    Vector3D pos=Position();
    pos.z=0;
    float R=(pos-obj->Position()).length();
    //    ,     
    if((dr+R).length()<obj->Radius())
      if (obj->IncreaseDamage(bombDamage))
        airfield->GroundUnitDestroyed();
    //    
    else if((dr+R).length()<obj->Radius()+r_bomber)
    {
      float RR=obj->Radius()-(dr+R).length();
      if(obj->IncreaseDamage(bombDamage*exp(-RR*bombRalpha)))
        airfield->GroundUnitDestroyed();
    }
  }
  target=0;
}

void EnemyBomber::GoHome()
{
  target=0;
  if(event_id) dispatcher->DeleteEvent(event_id);
  event_id=0;
  GoTo(airfield->Position());
  event_id=dispatcher->AddEvent(points.back().time-dispatcher->GetTime(), &OnBackHome, this, 0, "back_home");
  gohome=1;
  bMovingAside=0;

  if(fighters.get())
  {
    for(size_t i=0;i<fighters->size();i++)
      (*fighters)[i]->GoHome();
    delete fighters.release();
  }
}

void EnemyBomber::SetFighters(PFighterList p)
{
  fighters=p;
  for(size_t i=0;i<fighters->size();i++)
    (*fighters)[i]->SetBomber(this);
}

void EnemyBomber::DrawSelection(IDrawing* draw)
{
  Aircraft::DrawSelection(draw);
  if(target)
    draw->drawTargetSelection(target, 0xff);
}


////////////////////////////////////////////////////////////////
//                                                            //
//                TDefendingFighter                           //
//                                                            //
////////////////////////////////////////////////////////////////
DefendingFighter::DefendingFighter(DefendingAirfield* da, TTH &_tth)
                        : Aircraft(0, 0, _tth), airfield(da)
{
  target=0;
  ready=1;
	position=airfield->Position();
  flying=0;

  rocketV=tth.get("rocketV")/TIME_FACTOR;
  maxRocketT=tth.get("maxRocketT")*TIME_FACTOR;
  rocketProb=tth.get("rocketProb");
  rocketR=tth.get("rocketR");
  rockets=rocketsMax=tth.get("rocketsMax");
  fighterBattleR=tth.get("fighterBattleR");
  fighterV=tth.get("fighterV")/TIME_FACTOR;
  minH=tth.get("minH");
  maxH=tth.get("maxH");
  rocketRmax=tth.get("rocketRmax");
  rocketRalpha=tth.get("rocketRalpha");
}

int DefendingFighter::AssignTarget(Radar &r, TMovingObject &object, const Vector3D &pos, const Vector3D &sp)
{
  radar=&r;
//  warlog::puts("AssigntTarget start");

  Vector3D tp=position-pos;
  float d=tp.length();
//           
  if(d<DIV_EPS)
     return 0;
  float bomberV=sp.length();

  if(bomberV<DIV_EPS) return 0;
  float cosb=(tp&sp)/d/bomberV;

//,       - .
//      
  float Ddiv4=d*d*(rocketV*rocketV+bomberV*bomberV*(cosb*cosb-1));
  float dt=0;
  if(Ddiv4>0)
    dt=(-bomberV*d*cosb+sqrt(Ddiv4))/(rocketV*rocketV-bomberV*bomberV);
//  ,      
  if(Ddiv4<0 || dt>maxRocketT)
  {
    float a=fighterV*fighterV-bomberV*bomberV;
    if(a<DIV_EPS)
      return 0;
    float b_2=(rocketV-fighterV)*fighterV*maxRocketT+bomberV*d*cosb;
    float c=(rocketV*rocketV-fighterV*fighterV)*maxRocketT*maxRocketT-d*d;

    Ddiv4=b_2*b_2-a*c;
    if(Ddiv4<0)
      return 0;
    Ddiv4=sqrt(Ddiv4);
    dt=(-b_2+Ddiv4)/a;
    if((dt-maxRocketT)*fighterV>fighterBattleR)
          return 0;
  }
  if(dt<0)
      return 0;
  //       
  targetEndPos=pos+sp*dt;
  float launchRocketTime=dt-maxRocketT;
  rocketFlightTime=maxRocketT;
  if(launchRocketTime<0)
  {
    rocketFlightTime=dt;
    launchRocketTime=0;
  }
  dispatcher->AddEvent(launchRocketTime, &OnLaunchRocket, this, 0, "launch_rocket");

  //  
  if(!flying)
  {
    speed=Vector3D(1,0,0);
    position+=Vector3D(0, 0, minH+Random()*(maxH-minH));
    speed*=fighterV;
  }
  GoTo(targetEndPos);
  ready=0;
  flying=1;
  target=&object;

  return 1;
}

//  
void __stdcall DefendingFighter::OnLaunchRocket(WarObject* __this, int)
{
  DefendingFighter* _this=dynamic_cast<DefendingFighter*>(__this);
  if(!_this->target) return;
  dispatcher->AddEvent(_this->rocketFlightTime, &OnRocketFire, _this, 0, "launch_rocket");
  _this->rocket=new TRocket(_this->targetEndPos, _this->position, _this->rocketV, _this->rocketFlightTime*2+10);
}

// 
void __stdcall DefendingFighter::OnRocketFire(WarObject* __this, int)
{
    DefendingFighter* _this=dynamic_cast<DefendingFighter*>(__this);
    if(!_this->target) return;

    // 
    Vector3D rocketPos=_this->rocket->Position();
    delete _this->rocket;

    _this->rocket=0;
    _this->rockets--;

    _this->airfield->RocketFire();

   // ?
    int success = 0;


    //    ,      rocketRmax
    for (WarObjectsList::iterator i = WarObjects.begin(); i != WarObjects.end(); )
    {
        MovingObject *obj = dynamic_cast<MovingObject *>(*i);
        i++;
        if (!obj ||obj==_this|| typeid(*obj) == typeid(Rocket) || (obj->Position() - rocketPos).length() > _this->rocketRmax)
                continue;

        //   
        float R = (obj->Position() - rocketPos).length();
        float P = R < _this->rocketR ? _this->rocketProb :
                  _this->rocketProb*exp(-(R-_this->rocketR)*_this->rocketRalpha);
        if (Random() < P) // !!! 
        {
                if (obj == _this->target)
                        success = 1;
                _this->airfield->AirUnitDestroyed();
                delete obj;
        }
    }

	//   ...
    bool bRadarPresent=false;
    for (WarObjectsList::const_iterator i = WarObjects.begin(); i != WarObjects.end(); i++)
      if (*i == _this->radar)
      {
        bRadarPresent=1;
        break;
      }

    if(bRadarPresent)
    {
	    if (_this->target != NULL && !success && _this->rockets>0)
	    {
	        //    ,  
	        //     
	        Vector3D pos, sp;
	        if (_this->radar->GetObjectCoords(_this->target, pos, sp))
	        {
	            if(_this->AssignTarget(*_this->radar, *_this->target, pos, sp))
                return;
	        }
	    }
      //      
	    if (!success)
              _this->radar->AfterFire(_this->target);
		}
    _this->GoHome();
}

// 
void DefendingFighter::GoHome()
{
  target=0;
  GoTo(airfield->Position());
  dispatcher->AddEvent(points.back().time-dispatcher->GetTime(), &OnBackHome, this, 0, "back_home");

}

// 
void __stdcall DefendingFighter::OnBackHome(WarObject* __this, int)
{
  DefendingFighter* _this=dynamic_cast<DefendingFighter*>(__this);
  _this->ready=1;
  _this->rockets=_this->rocketsMax;
  _this->speed=0;
  _this->flying=0;
  _this->position.z=0;
  while(!_this->points.empty()) _this->points.pop();
  _this->airfield->OnFighterReturn();
}

void DefendingFighter::DrawSelection(IDrawing* draw)
{
  Aircraft::DrawSelection(draw);
  if(target)
    draw->drawTargetSelection(target, 0xff);
}


void DefendingFighter::OnDeleteObject(WarObject *obj)
{
  if(target==obj)
    GoHome();
}



////////////////////////////////////////////////////////////////
//                                                            //
//                AccompFighter                              //
//                                                            //
////////////////////////////////////////////////////////////////
AccompFighter::AccompFighter(BomberAirfield* ba, TTH &_tth)
                        : Aircraft(0, 0, _tth), airfield(ba)
{
  ready=1;
	position=airfield->Position();
  flying=0;
  bomber=0;
  rocketV=tth.get(rocketV)/TIME_FACTOR;
  maxRocketT=tth.get(maxRocket)*TIME_FACTOR;
  rocketProb=tth.get(rocketProb);
  rocketR=tth.get(rocketR);
  rockets=rocketsMax=tth.get(rocketsMax);
  rocketRmax=tth.get(rocketRmax);
  rocketRalpha=tth.get(rocketRalpha);
  ViewTime=tth.get(ViewTime)*TIME_FACTOR;
  pos_sigma=Vector3D(tth.get(pos_sigma));
  speed_sigma=Vector3D(tth.get(speed_sigma)/TIME_FACTOR);
}

//    
//   
void __stdcall AccompFighter::OnLaunchRocket(WarObject* __this, int)
{
  AccompFighter* _this=dynamic_cast<AccompFighter*>(__this);
  if(!_this->rockets) return;
  float mindt;
  DefendingFighter* target=0;

  // ,    
  for (WarObjectsList::iterator i = WarObjects.begin(); i != WarObjects.end(); )
  {
        DefendingFighter *obj = dynamic_cast<DefendingFighter *>(*i);
        i++;
        if (!obj)
                continue;

        Vector3D tp=_this->position-obj->Position();
        float d=tp.length();

        if(d<DIV_EPS)
          continue;

        float targetV=obj->Speed().length();
        if(targetV<DIV_EPS)
          continue;
        float cosb=(tp&obj->Speed())/d/targetV;

        float Ddiv4=d*d*(_this->rocketV*_this->rocketV+targetV*targetV*(cosb*cosb-1));
        if(Ddiv4<0) continue;
        float a=(_this->rocketV*_this->rocketV-targetV*targetV);
        if(a<DIV_EPS) continue;
        float dt=(-targetV*d*cosb+sqrt(Ddiv4))/a;
        if((!target||dt<mindt)&&(dt<_this->maxRocketT))
        {
          target=obj;
          mindt=dt;
        }
  }
//   ,     ,    
//  
  if(!target)
    dispatcher->AddEvent(_this->ViewTime, &OnLaunchRocket, _this, 0, "launch_rocket");
  else
  {
    //     
    Vector3D endTargetPos=target->Position()+norm_raspr(_this->pos_sigma)+(target->Speed()+norm_raspr(_this->speed_sigma))*mindt;
    // 
    _this->rocket=new Rocket(endTargetPos, _this->position, _this->rocketV, mindt*2+10);
    dispatcher->AddEvent(mindt, &OnRocketFire, _this, 0, "");
  }
}

// 
void __stdcall AccompFighter::OnRocketFire(WarObject* __this, int)
{
    AccompFighter* _this=dynamic_cast<AccompFighter*>(__this);
    if(!_this->rocket)
    {
      if(_this->rockets)
       dispatcher->AddEvent(_this->ViewTime, &OnLaunchRocket, _this, 0, "launch_rocket");
      return;
    }
    // 
    Vector3D rocketPos=_this->rocket->Position();
    delete _this->rocket;
    _this->rocket=0;
    _this->rockets--;

    //    ,      rocketRmax
    for (WarObjectsList::iterator i = WarObjects.begin(); i != WarObjects.end(); )
    {
        MovingObject *obj = dynamic_cast<MovingObject *>(*i);
        i++;
        if (!obj || obj==_this||typeid(*obj) == typeid(Rocket) || (obj->Position() - rocketPos).length() > _this->rocketRmax)
                continue;
        //   
        float R = (obj->Position() - rocketPos).length();
        float P = R < _this->rocketR ? _this->rocketProb :
                  _this->rocketProb*exp(-(R-_this->rocketR)*_this->rocketRalpha);
        if (Random() < P) // !!! 
        {
          _this->airfield->AirUnitDestroyed();
          delete obj;
        }
    }
    if(_this->rockets)
      dispatcher->AddEvent(_this->ViewTime, &OnLaunchRocket, _this, 0, "launch_rocket");
}

// 
void TAccompFighter::GoHome()
{
  bomber=0;
  GoTo(airfield->Position());
  dispatcher->AddEvent(points.back().time-dispatcher->GetTime(), &OnBackHome, this, 0, "back_home");
}

// 
void __stdcall AccompFighter::OnBackHome(WarObject* __this, int)
{
  AccompFighter* _this=dynamic_cast<AccompFighter*>(__this);
  _this->ready=1;
  _this->rockets=_this->rocketsMax;
  _this->speed=0;
  _this->flying=0;
  _this->position.z=0;
  while(!_this->points.empty()) _this->points.pop();
  _this->airfield->OnFighterReturn();
}

//   
void AccompFighter::SetBomber(EnemyBomber* b)
{
  bomber=b;
  ready=0;
  flying=1;
  if(bomber == 0)
    GoHome();
  else
    dispatcher->AddEvent(ViewTime, &OnLaunchRocket, this, 0, "launch_rocket");
}

void AccompFighter::DrawSelection(IDrawing* draw)
{
  Aircraft::DrawSelection(draw);
}

void AccompFighter::OnDeleteObject(WarObject *obj)
{
  if(bomber==obj)
    GoHome();
}

void AccompFighter::Draw(IDrawing* draw)
{
  Vector3D s=speed;
  if(speed.length()>DIV_EPS)
        s.normalize();
  else
        return;
  s*=GetScreenSize();
  draw->line(GetColor(), position, position+Vector3D(s.x,s.y,0));
}
