#include "stdafx.h"
#include "objectbutton.h"

BEGIN_MESSAGE_MAP(CItemButton, CWnd)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_PAINT()
END_MESSAGE_MAP()

CItemButton::CItemButton() :
m_watch_mouse(false),
BUTTON_WIDTH(100),
BUTTON_HEIGHT(100)
{
}

void CItemButton::Create(double x, double y, CWnd *parent)
{
	RECT button_pos;

	button_pos.left = LONG(GetX(x) - BUTTON_WIDTH/2);
	button_pos.top = LONG(GetY(y) - BUTTON_HEIGHT/2);
	button_pos.right = button_pos.left + BUTTON_WIDTH;
	button_pos.bottom = button_pos.top + BUTTON_HEIGHT;

	((CWnd *)this)->Create(NULL, "Button", WS_CHILD|WS_VISIBLE, button_pos, parent, 0);
	GetDC()->SetBkMode(TRANSPARENT);
}

double CItemButton::GetXPos(void)
{
	RECT rc;
	RECT rc_parent;
	GetWindowRect(&rc);
	GetParent()->GetWindowRect(&rc_parent);

	return (rc.left - rc_parent.left - GetSystemMetrics(SM_CXFRAME) - GetX(0.0) + BUTTON_WIDTH/2)/ GetLengthX(1.0);
}

double CItemButton::GetYPos(void)
{
	RECT rc;
	RECT rc_parent;
	GetWindowRect(&rc);
	GetParent()->GetWindowRect(&rc_parent);

	return -(rc.top - rc_parent.top - GetSystemMetrics(SM_CYFRAME) - GetSystemMetrics(SM_CYCAPTION) - GetY(0.0) + BUTTON_HEIGHT/2)/ GetLengthY(1.0);
}

CItemButton::~CItemButton()
{
}

void CItemButton::OnLButtonDown(UINT nFlags, CPoint point)
{
	m_watch_mouse = true;
	m_mouse_point = point;
}

void CItemButton::OnMouseMove(UINT nFlags, CPoint point)
{
	RECT rc;
	RECT rc_parent;
	RECT new_pos;

	if(!m_watch_mouse) return;
	CPoint diff = point - m_mouse_point;
	
	GetWindowRect(&rc);
	GetParent()->GetWindowRect(&rc_parent);

	new_pos.left = rc.left - rc_parent.left - GetSystemMetrics(SM_CXFRAME) + diff.x;
	new_pos.top = rc.top - rc_parent.top - GetSystemMetrics(SM_CYFRAME) - GetSystemMetrics(SM_CYCAPTION) + diff.y;
	new_pos.right = new_pos.left + rc.right - rc.left;
	new_pos.bottom = new_pos.top + rc.bottom - rc.top;
	MoveWindow(new_pos.left, new_pos.top, new_pos.right - new_pos.left, new_pos.bottom - new_pos.top);
	RedrawWindow();
}

void CItemButton::OnLButtonUp(UINT nFlags, CPoint point)
{
	m_watch_mouse = false;
}

void CItemButton::OnPaint()
{
	PAINTSTRUCT ps;
	CDC dc;

	BeginPaint(&ps);
	dc.Attach(ps.hdc);
	dc.FillSolidRect(&ps.rcPaint, RGB(100, 100, 100));
	dc.Detach();
	EndPaint(&ps);
}

BEGIN_MESSAGE_MAP(CObjectButton, CItemButton)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CObjectButton::CObjectButton()
{
	BUTTON_WIDTH = 16;
	BUTTON_HEIGHT = 16;
}

void CObjectButton::OnPaint()
{
	PAINTSTRUCT ps;
	CDC dc;

	BeginPaint(&ps);
	dc.Attach(ps.hdc);
	dc.SetBkMode(TRANSPARENT);

	CBrush br(RGB(0, 128, 0)), *def_br;
	CPen pen, *def_pen;
	pen.CreatePen(PS_SOLID, 1, RGB(0, 255, 0));

	RECT rect;
	GetClientRect(&rect);
	dc.FillSolidRect(&ps.rcPaint, RGB(0, 0, 0));
	rect.top = (rect.top + rect.bottom)/2 - 4;
	rect.bottom = rect.top + 8;
	rect.left = (rect.left + rect.right)/2 - 4;
	rect.right = rect.left + 8;
	def_br = dc.SelectObject(&br);
	def_pen = dc.SelectObject(&pen);
	dc.Rectangle(&rect);
    dc.SelectObject(def_pen);
    dc.SelectObject(def_br);
	br.DeleteObject();
	pen.DeleteObject();

	dc.Detach();
	EndPaint(&ps);
}

BEGIN_MESSAGE_MAP(CMCZRKButton, CItemButton)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CMCZRKButton::CMCZRKButton() 
{
	BUTTON_WIDTH = 50;//2*GetLengthX(Zrk1Multi.maxR);
	BUTTON_HEIGHT = 50;//2*GetLengthY(Zrk1Multi.maxR);
}

void CMCZRKButton::OnPaint()
{
	PAINTSTRUCT ps;
	CDC dc;

	BeginPaint(&ps);
	dc.Attach(ps.hdc);

	CBrush br;
	CBrush *def_br;
	CPen pen;
	CPen *def_pen;
	double a, g, r;

	pen.CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
	def_pen = dc.SelectObject(&pen);
	br.CreateSolidBrush(RGB(0, 0, 128));
	def_br = dc.SelectObject(&br);

	RECT rect;
	GetClientRect(&rect);
	dc.FillSolidRect(&ps.rcPaint, RGB(0, 0, 0));

	int center_x = (rect.left + rect.right)/2;
	int center_y = (rect.top + rect.bottom)/2;

	rect.top = (rect.top + rect.bottom)/2 - 4;
	rect.bottom = rect.top + 8;
	rect.left = (rect.left + rect.right)/2 - 4;
	rect.right = rect.left + 8;

	dc.Ellipse(&rect);

	dc.SelectObject(def_br);
	br.DeleteObject();
	//   
	r = Zrk1Multi.maxR;

	GetClientRect(&rect);
	rect.top = LONG((rect.top + rect.bottom)/2 - GetLengthY(r));
	rect.bottom = LONG(rect.top + 2*GetLengthY(r));
	rect.left = LONG((rect.left + rect.right)/2 - GetLengthX(r));
	rect.right = LONG(rect.left + 2*GetLengthX(r));

	a = 0;
	g = Zrk1Multi.gamma * PI / 360.;

	POINT start;
	POINT end;
	start.x = LONG(center_x + 10*cos(a - g));
	start.y = LONG(center_y - 10*sin(a - g));
	end.x = LONG(center_x + 10*cos(a + g));
	end.y = LONG(center_y - 10*sin(a + g));

	br.CreateSolidBrush(RGB(0, 0, 128));
	def_br = dc.SelectObject(&br);

	dc.Pie(&rect, start, end);

	dc.SelectObject(def_br);
	br.DeleteObject();
	dc.SelectObject(def_pen);
	pen.DeleteObject();

	dc.Detach();
	EndPaint(&ps);
}

BEGIN_MESSAGE_MAP(CSCZRKButton, CItemButton)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CSCZRKButton::CSCZRKButton() 
{
	BUTTON_WIDTH = 50;//2*GetLengthX(Zrk2Single.maxR);
	BUTTON_HEIGHT = 50;//2*GetLengthY(Zrk2Single.maxR);
}

void CSCZRKButton::OnPaint()
{
	PAINTSTRUCT ps;
	CDC dc;

	BeginPaint(&ps);
	dc.Attach(ps.hdc);

	CBrush br;
	CBrush *def_br;
	CPen pen;
	CPen *def_pen;
	double a, g, r;

	pen.CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
	def_pen = dc.SelectObject(&pen);
	br.CreateSolidBrush(RGB(0, 0, 128));
	def_br = dc.SelectObject(&br);

	RECT rect;
	GetClientRect(&rect);
	dc.FillSolidRect(&ps.rcPaint, RGB(0, 0, 0));

	int center_x = (rect.left + rect.right)/2;
	int center_y = (rect.top + rect.bottom)/2;

	rect.top = (rect.top + rect.bottom)/2 - 4;
	rect.bottom = rect.top + 8;
	rect.left = (rect.left + rect.right)/2 - 4;
	rect.right = rect.left + 8;

	dc.Ellipse(&rect);

	dc.SelectObject(def_br);
	br.DeleteObject();
	
	r = Zrk2Single.maxR;

	GetClientRect(&rect);
	rect.top = (rect.top + rect.bottom)/2 - GetLengthY(r);
	rect.bottom = rect.top + 2*GetLengthY(r);
	rect.left = (rect.left + rect.right)/2 - GetLengthX(r);
	rect.right = rect.left + 2*GetLengthX(r);

	a = 0;
	g = Zrk2Single.gamma * PI / 360.;

	POINT start;
	POINT end;
	start.x = center_x + 10*cos(a - g);
	start.y = center_y - 10*sin(a - g);
	end.x = center_x + 10*cos(a + g);
	end.y = center_y - 10*sin(a + g);

	br.CreateSolidBrush(RGB(0, 0, 128));
	def_br = dc.SelectObject(&br);

	dc.Pie(&rect, start, end);

	dc.SelectObject(def_br);
	br.DeleteObject();
	dc.SelectObject(def_pen);
	pen.DeleteObject();

	dc.Detach();
	EndPaint(&ps);
}

BEGIN_MESSAGE_MAP(CFAIRButton, CItemButton)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CFAIRButton::CFAIRButton() 
{
	BUTTON_WIDTH = 16;
	BUTTON_HEIGHT = 16;
}

void CFAIRButton::OnPaint()
{
	PAINTSTRUCT ps;
	CDC dc;

	CPen pen;
	CBrush brush;
	CPen* def_pen;
	CBrush* def_brush;

	BeginPaint(&ps);
	dc.Attach(ps.hdc);

	pen.CreatePen(PS_SOLID, 1, RGB(255, 0, 0));
	brush.CreateSolidBrush(RGB(100, 0, 0));
	def_pen = dc.SelectObject(&pen);
	def_brush = dc.SelectObject(&brush);

	RECT rect;
	GetClientRect(&rect);
	dc.FillSolidRect(&ps.rcPaint, RGB(0, 0, 0));

	int center_x = (rect.left + rect.right)/2;
	int center_y = (rect.top + rect.bottom)/2;

	rect.top = (rect.top + rect.bottom)/2 - 3;
	rect.bottom = rect.top + 6;
	rect.left = (rect.left + rect.right)/2 - 3;
	rect.right = rect.left + 6;

	dc.Ellipse(&rect);

    dc.SelectObject(def_brush);
    brush.DeleteObject();
    dc.SelectObject(def_pen);
    pen.DeleteObject();

    dc.Detach();
	EndPaint(&ps);
}

BEGIN_MESSAGE_MAP(CBAIRButton, CItemButton)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CBAIRButton::CBAIRButton() 
{
	BUTTON_WIDTH = 16;
	BUTTON_HEIGHT = 16;
}

void CBAIRButton::OnPaint()
{
	PAINTSTRUCT ps;
	CDC dc;

	CPen pen;
	CBrush brush;
	CPen* def_pen;
	CBrush* def_brush;

	BeginPaint(&ps);
	dc.Attach(ps.hdc);

	pen.CreatePen(PS_SOLID, 1, RGB(255, 0, 0));
	brush.CreateSolidBrush(RGB(100, 0, 0));
	def_pen = dc.SelectObject(&pen);
	def_brush = dc.SelectObject(&brush);

	RECT rect;
	GetClientRect(&rect);
	dc.FillSolidRect(&ps.rcPaint, RGB(0, 0, 0));

	int center_x = (rect.left + rect.right)/2;
	int center_y = (rect.top + rect.bottom)/2;

	rect.top = (rect.top + rect.bottom)/2 - 4;
	rect.bottom = rect.top + 8;
	rect.left = (rect.left + rect.right)/2 - 4;
	rect.right = rect.left + 8;

	dc.Ellipse(&rect);

    dc.SelectObject(def_brush);
    brush.DeleteObject();
    dc.SelectObject(def_pen);
    pen.DeleteObject();

    dc.Detach();
	EndPaint(&ps);
}

BEGIN_MESSAGE_MAP(CAIRButton, CItemButton)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CAIRButton::CAIRButton() 
{
	BUTTON_WIDTH = 16;
	BUTTON_HEIGHT = 16;
}

void CAIRButton::OnPaint()
{
	PAINTSTRUCT ps;
	CDC dc;

	CPen pen;
	CBrush brush;
	CPen* def_pen;
	CBrush* def_brush;

	BeginPaint(&ps);
	dc.Attach(ps.hdc);

	pen.CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
	brush.CreateSolidBrush(RGB(0, 0, 128));
	def_pen = dc.SelectObject(&pen);
	def_brush = dc.SelectObject(&brush);

	RECT rect;
	GetClientRect(&rect);
	dc.FillSolidRect(&ps.rcPaint, RGB(0, 0, 0));

	int center_x = (rect.left + rect.right)/2;
	int center_y = (rect.top + rect.bottom)/2;

	rect.top = (rect.top + rect.bottom)/2 - 4;
	rect.bottom = rect.top + 8;
	rect.left = (rect.left + rect.right)/2 - 4;
	rect.right = rect.left + 8;

	dc.Ellipse(&rect);

    dc.SelectObject(def_brush);
    brush.DeleteObject();
    dc.SelectObject(def_pen);
    pen.DeleteObject();

    dc.Detach();
	EndPaint(&ps);
}